@@ -1187,6 +1187,10 @@ static void release_dev(struct file * filp)
 		}
 	}
 #endif
+
+	if (tty->driver.close)
+		tty->driver.close(tty, filp);
+
 	/*
 	 * Sanity check: if tty->count is going to zero, there shouldn't be
 	 * any waiters on tty->read_wait or tty->write_wait.  We test the
@@ -1244,9 +1248,6 @@ static void release_dev(struct file * filp)
 	 * block, so it's safe to proceed with closing.
 	 */
 
-	if (tty->driver.close)
-		tty->driver.close(tty, filp);
-
 	if (pty_master) {
 		if (--o_tty->count < 0) {
 			printk("release_dev: bad pty slave count (%d) for %s\n",
@@ -2072,9 +2073,6 @@ int tty_init(void)
 #ifdef CONFIG_RISCOM8
 	riscom8_init();
 #endif
-#ifdef CONFIG_BAYCOM
-	baycom_init();
-#endif
 #ifdef CONFIG_SPECIALIX
 	specialix_init();
 #endif
@@ -2082,4 +2080,3 @@ int tty_init(void)
 	vcs_init();
 	return 0;
 }
-
